{{
        Node Interface API

        Author: Shawn Standfast
        Date:    7/7/2010
        Node_Interface.spin

        Description: Provides Serial Interface API for System Controller to Communicate with Addressable Nodes

}}

con
{
Define Constants for Message Commands
}
{-- ----------------------------------------------------------------------------

-- Message Format: Start Byte, Node Address, Command Byte, Data Length*, Data
--
-- *Data Length does not include header information, but does include checksum
-- -----------------------------------------------------------------------------

const byte startByte = 0xBB -- Indicates beginning of new message
}
  startByte = $BB

{
-- Change Source Use

-- Start Byte, Address, Change_Source, 0x02 , SOURCE, ChkSum
const byte change_source = 0xB0  -- Changes Node's Audio Source; SC <-> Node
}

   change_source = $B0

{
-- Update Node Use
-- Start Byte, 0x00, Update_Node, Data Length, SOURCE, PLAY STATUS, Now_Playing Data, ChkSum
const byte update_node = 0xB1  -- General Broadcast - Update all nodes with specified SOURCE; SC -> Node
}
   update_node = $B1
{
-- Change_baud Use
-- SB, 0xFF, Change_Baud, 0x03, BaudMSB, BaudLSB, ChkSum
const byte change_baud = 0xB2  -- General Boradcast - Change Baudrate of RS-422 Bus (Debug Only); SC -> Node
}
   change_baud = $B2
{
-- Change Volume Use
-- SB, Address, Change_Vol, 0x02, VOLUME, ChkSum
const byte change_Vol = 0xB3   -- Changes Node's Volume Setting; SC <-> Node
}
   change_Vol = $B3
{
-- SB, SC_Address, Play_next, 0x01, ChkSum
const byte play_next = 0xB4 -- advances playlist to next song; SC <- Node
}
   play_next = $B4
{
-- SB, SC_Address, Play, 0x01, ChkSum
const byte play = 0xB5 -- Starts the Media Player on PC; SC <- Node
}
   play = $B5
{
-- SB, SC_Address, Play_prev, 0x01, ChkSum
const byte play_prev = 0xB6 -- advances playlist to previous song; SC <- Node
}
   play_prev = $B6
{
-- SB, SC_Address, Pause, 0x01, ChkSum
const byte pause = 0xB7 -- Pauses playback; SC <- Node
}
   pause = $B7
{
-- SB, SC_Address, Stp, 0x01, ChkSum
const byte stp = 0xB8 -- Stops Playback; SC <- Node
}
   stp = $B8
{
-- SB, SC_Address, Controller_ok, 0x0, Source, MUTEState, Volume ChkSum
const byte controller_ok = 0xB9  -- Response that zone is ok; SC <- Node
}
   controller_ok = $B9
{
-- SB, Address, mute, 0x02, MUTEState, ChkSum; MUTEState = 1 for Mute, 0 for On
const byte mute = 0xBA  -- Set's Node's volume to zero (0) but retains previous level; SC <-> Node
}
   mute = $BA
{
-- SB, SC_Address, Play, 0x01, ChkSum
const byte ack = 0xAA  -- Node acknowledge response to controller of message received; SC <- Node
}
   ack = $AA
{
-- SB, SC_Address, Error, Data Count, Error Data, ChkSum
const byte ErrorData = 0xCC  -- Node Error has occured, Error data follows
}
   ErrorData = $CC
{
-- SB, SC_Address, RoomTemp, 0x04 , Room Temp High, Room Temp Low, Room Temp Setpoint, ChsSum
const byte RoomTemp = 0xBC -- Message passes Room Temperature Data and Setpoint Information; SC <-> Node
}
   RoomTemp = $BC

{
-- Fixed Node Addresses
const byte SC_Address = 0xFF --  System Controller Address
const byte BC_Address = 0x00 -- General Broadcast Address -- All Nodes should listen
}
    SC_Address = $FF
    BC_Address = $00
{
-- Message Component Indexes
const byte command_index = 0x02
const byte address_index = 0x01
const byte length_index = 0x03
const byte data_index = 0x04
}
   command_index = $02
   address_index = $01
   length_index = $03
   data_index = $04

con
{
Define Parameters for RS422 Setup
}
  
  baud = 9600
  tx_pin = 22
  rx_pin = 23

  timeout = 3 'amount of time to wait for a response from the node; value in seconds

obj {Include Child Objects}
  serial : "FullDuplexSerial"
  
var {Create Buffers and Read/Write Pointers for messages}
  byte ntx_buffer[256]
  byte nrx_buffer[256]
  byte tx_readptr
  byte tx_writeptr
  byte rx_readptr
  byte rx_writeptr

  byte ack_req
  byte updateBuffer[253]

var  {Parsed message variables}
  byte msg_info[2]

  
  
pub  init : success {{Initializes the object}} 

   bytefill(@ntx_buffer,0,255) 'Clear Transmit Buffer
   bytefill(@nrx_buffer,0,255) 'Clear Received Buffer

   tx_readptr := rx_readptr := tx_writeptr := rx_writeptr := $00  'Clear read and write pointers
   
   
   success := serial.start(rx_pin, tx_pin, 0, baud)

pub stop   {{Stops the object}}
  serial.rxflush
  serial.stop

pri addData(data)   {adds byte to tx buffer}
  ntx_buffer[tx_writeptr] := data.byte
  tx_writeptr := (tx_writeptr + 1) & $FF
    
pri send_message(ackreq) |t1,t2  {sends composed message to node}

  ack_req := ackreq
  tx_readptr := $00

  repeat
    serial.tx(ntx_buffer[tx_readptr])
  until (tx_writeptr == (tx_readptr := ((tx_readptr + 1) & $FF)))

  tx_writeptr := $00
  tx_readptr := $00

pub muteZone(address, mutestate) : ok | chksum {{Sends Mute Command to Zone @ Address.  See Command Definition for Mutestate valid states}}
  addData(startByte)
  addData(address)
  addData(mute)
  addData($02)
  addData(mutestate)
  chksum.byte := startByte + address + mute + $02 + mutestate
  addData(chksum)
  send_message(false)

  return ok := ReceiveMsg
  
pub getTemp(address) : temperature | Temp, LB, HB   {{Asks node @ Address for it's temperature value}}
{
-- SB, SC_Address, RoomTemp, 0x04 , Room Temp High, Room Temp Low, Room Temp Setpoint, ChsSum
}
  addData(startByte)
  addData(address)
  addData(roomTemp)
  addData($01)
  Temp := startByte + address + roomTemp + $01
  addData(Temp.byte)
  send_message($00)

  if ReceiveMsg < 0
    return temperature := -1
  else
    'Temperature data is stored in nrx_buffer as follows:  RAW A/D Room Temp High-Byte, RAW A/D Room Temp Low-byte, Room Temp Setpoint (in degrees)
'    display.out($00)

    
    temperature := ((nrx_buffer[rx_readptr ] << 8) + nrx_buffer[(rx_readptr + 1)])
'    display.out($0D)
'    display.out($0D)
'    display.str(string("Temperature : "))
'    display.hex(temperature,4)
    return temperature
    
pub updateNode(datapointer, len) | i, chksum, temp {{Sends General Broadcast to update all nodes with appropriately selected source with Now Playing data}}

  i := 0
  chksum := 0
  addData(startByte)
  temp := BC_Address
  addData(temp.byte)
  addData(update_node)
  addData(len)
  chksum.byte := startByte + BC_address + update_node + len
  repeat i from 0 to len - 2
    temp := byte[datapointer][i]
    addData(temp.byte)
    chksum.byte += temp
  addData(chksum.byte)
  send_message($00)

pub changeSource(address, source) : ok | chksum  {{Sends command to zone @ address to change source display to Source}}
  addData(startByte)
  addData(address)
  addData(change_source)
  addData($02)
  addData(source.byte)
  chksum.byte := startByte + address + change_source + $02 + source
  addData(chksum)
  send_message($00)

  return ok := ReceiveMsg

pub changeVolume(address, volume) : ok | chksum {{Sends volume information to zone @ address}}
  addData(startByte)
  addData(address)
  addData(change_Vol)
  addData($02)
  addData(volume.byte)
  chksum.byte := startbyte + address + change_Vol + $02 + volume
  addData(chksum)

  send_message($00)

  return ok := receiveMsg
      
pub rx | Temp       {{Debug only}}
  'return Temp := serial.rxcheck
  return Temp := ReceiveMsg

pub tx(data)   {{Debug only}}
  serial.tx(data)

obj    {Debug Only}
'  display : "VGA_Text"
'pub startdisp
'  display.start(16)
'  display.setcolors(@colorpallet2)
    
pri ReceiveMsg : MsgValid | t, t1, t2, t3, sec_cnt, sb_rec, bytecount, len, error, chksum  {Waits up to "timeout" for response from zone and stores data into nrx_buffer and msg_info}
  ' Initialize Routine
  error := False  'Indicates an error with reception; either timeout or buffer overflow
  len := -1     'Contains the length parameter of the message;  initialize to -1 so that it doesn't equal bytecount
  bytecount := $00  'Counts the number of bytes received
  sec_cnt := $00   'Counts the number of seconds since last byte received
  sb_rec := false  'Indicates a valid start byte has been received
  MsgValid := $00
  chksum := $00    
  if ((rx_writeptr + 1) == rx_readptr) 'Only try and receive data when there is room in the buffer
    return -1
'  display.out($00)

  t1 := cnt  'grab start time
  
  repeat

    t := serial.rxtime(1)
    if t => 0   'check for data available
      if (t == startByte and sb_rec == false)   'is data a startbyte?  Are we already receiveing a message?
'        display.str(string("StartByte Received"))
'        display.out($0D)
        sb_rec := True                         'set startbyte received flag
        chksum.byte := startByte
      elseif (sb_rec ==  True)  'are we currently receiving a message?
'  '        display.str(string("sb_rec := true"))
'  '        display.out($0D)
        case bytecount
          0 : if t <> $FF   'Does Address Byte Match Controller Address?
'                display.str(string("Address Does Not Match"))
                sb_rec := false
                error := -1
              else
'                display.str(string("Address Matches"))
'                display.out($0D)
                bytecount += 1
                chksum.byte += t.byte
          1 : msg_info[0] := t.byte 'Store Command Byte
'              display.str(string("Command Byte : "))
'              display.hex(t.byte,2)
'              display.out($0D)
              bytecount += 1
              chksum.byte += t.byte
          2 : msg_info[1] := t.byte 'Store Length Data
'              display.str(string("Length Byte : "))
'              display.hex(t.byte,2)
'              display.out($0D)
              len := t.byte + 3' Save Length information
              bytecount += 1
              chksum.byte += t.byte
              rx_readptr := rx_writeptr
          other :
                if bytecount => 3  ' Only save data when we are receiving a message 
                  nrx_buffer[rx_writeptr] := t.byte  'save data
'                  display.str(string("Data Byte, Write PTR : "))
'                  display.hex(nrx_buffer[rx_writeptr],2)
'                  display.str(string(" "))
'                  display.hex(rx_writeptr,2)
'                  display.out($0D)
                  bytecount += 1  'incriment bytes received
                  rx_writeptr := (rx_writeptr + 1)' & $FF  'incriment write pointer
                  if bytecount == len
                    if chksum.byte <> t.byte
                      error := -1
'                      display.str(string("Invalid checksum"))
'                      display.out($0D)
'                      display.str(string("Received chksum : "))
'                      display.hex(t.byte,2)
'                      display.out($0D)
'                      display.str(string("Calculated chksum : "))
'                      display.hex(chksum.byte,2)
'                      display.out($0D)
'                    else
'                      display.str(string("Checksum Matches : "))
'                      display.hex(chksum.byte,2)
'                      display.out($0D)  
                  else
                    chksum.byte += t.byte
          
                 if ((rx_writeptr) == rx_readptr and bytecount <> len)  'check for buffer overflow; only overflow if not a full message received
                   error := -1
      sec_cnt := $00 'ditto
      t1 := cnt      'serial activity detected; reset timeout counter
    else   'no serial activity; count idle time
'      display.dec(cnt - t1)
'      display.out($0D)
'      t1 := cnt
      t2 :=  (cnt - (t1 + clkfreq)) #> 0
      sec_cnt += t2 <# 1
'      display.str(string("sec_cnt := "))
'      display.dec(sec_cnt)
'      display.out($0D)
      if sec_cnt => timeout ' check for timeout period expired
'        display.str(string("Timeout occured"))
'        display.out($0D)
        error := -1
      if t2
        t1 := cnt
  until ((bytecount == len) or error == -1)  'continue receiving until end of message or error
  sb_rec := false
  MsgValid := error
'  display.str(string("Error State is : "))
'  display.dec(MsgValid)
'  display.out($0D)
  return MsgValid
   
{dat
  colorpallet2          '        fore   back
                        '         RGB    RGB
                        byte    %%333, %%003    '0    white / blue
                        byte    %%000, %%110    '1   yellow / brown
                        byte    %%202, %%000    '2  magenta / black
                        byte    %%111, %%333    '3     grey / white
                        byte    %%033, %%011    '4     cyan / dark cyan
                        byte    %%020, %%232    '5    green / gray-green
                        byte    %%333, %%202    '6     white / magenta
                        byte    %%310, %%000    '7     orange/black
}