obj
  DHCP : "DHCP_SPI"
  W5100 : "Brilldea_W5100_SPI_Driver_Ver006.spin"
'  pst : "Parallax Serial Terminal.spin"
  STR : "STREngine"
  math : "FloatMath"
  
con

  '***************************************
  ' System Definitions     
  '***************************************

  _OUTPUT       = 1             'Sets pin to output in DIRA register
  _INPUT        = 0             'Sets pin to input in DIRA register  
  _HIGH         = 1             'High=ON=1=3.3V DC
  _ON           = 1
  _LOW          = 0             'Low=OFF=0=0V DC
  _OFF          = 0
  _ENABLE       = 1             'Enable (turn on) function/mode
  _DISABLE      = 0             'Disable (turn off) function/mode

  'I2C Interface
  _I2C_scl      = 28            'Output for the I2C serial clock
  _I2C_sda      = 29            'Input/output for the I2C serial data  

  '***************************************
  ' I2C Definitions
  '***************************************
  _EEPROM0_address = $A0        'Slave address of EEPROM

  CS = 16  ''CS: Clear Screen      
  CE = 11  ''CE: Clear to End of line     
  CB = 12  ''CB: Clear lines Below 

  HM =  1  ''HM: HoMe cursor       
  PC =  2  ''PC: Position Cursor in x,y          
  PX = 14  ''PX: Position cursor in X         
  PY = 15  ''PY: Position cursor in Y         

  NL = 13  ''NL: New Line        
  LF = 10  ''LF: Line Feed       
  ML =  3  ''ML: Move cursor Left          
  MR =  4  ''MR: Move cursor Right         
  MU =  5  ''MU: Move cursor Up          
  MD =  6  ''MD: Move cursor Down
  TB =  9  ''TB: TaB          
  BS =  8  ''BS: BackSpace          
           
  BP =  7  ''BP: BeeP speaker          

  ' return codes
  SUCCESS               = 0
  SOCKET_FAILED_OPENING = 1
  SERVER_TIMEOUT        = 2
  SERVER_NAK            = 3

  BUFFER_SIZE = 2048
  HostNameLen = 64
  DomainNameLen = 64

  HTTP_REQUEST_GET = 1
  HTTP_REQUEST_HEAD = 2
  HTTP_REQUEST_POST = 3

  HTTP_TIMEOUT = 80_000_000 * 4  ' 4 seconds, assumes 80Mhz clkfreq

  f_lsb = 0.1220703125

var
  long webserver_cog
  long lock_id
  long times_served
  byte temp_wanted
  long temperature[8]
  byte MAC_Address[6]
  long RequestType

  long  localSocket             '1 element for the socket number

  'Variables to info for where to return the data to
  byte  destIP[4]               '4 element array containing IP address ex. "192.168.0.16"
  long  destSocket              '1 element for the socket number

  long DHCP_Success
  byte IP[4]
  byte SubnetMask[4]
  byte GatewayIP[4]

  long DHCP_LeaseTime           ' duration of lease in seconds
  byte DHCP_Server[4]
  byte DNS_Server[4]

  byte DomainName[DomainNameLen]          ' you may need to make this larger if you have a very long domain name

  byte Buffer[BUFFER_SIZE]

  long stack[50]
    
pub UPNP_Start(time_ptr, debug_ptr) | temp, index

'  pst.start(19200)
  stop
  
  MAC_Address[0] := $00
  MAC_Address[1] := $08 '$08
  MAC_Address[2] := $DC '$DC
  MAC_Address[3] := $01 '$16
  MAC_Address[4] := $02 '$EF
  MAC_Address[5] := $06 '$13

'  pst.str(string("Starting DHCP..."))
'  pst.char($0D)
  DHCP_Success := DHCP.Start(@MAC_Address[0], @buffer[0])  'Start DHCP Service

  result := obtain_IP(time_ptr) 'Try to get an IP address from the DHCP server

  if result == (-1)   'Check for success, if DHCP fails, load default IP address values
    SetDefaultConfig                       
'    return -2

  DHCP.stop  'Stop DHCP service so we can create a W5100 driver accessable from this Object level
'  pst.str(string("DHCP Complete"))
'  pst.char($0D)
  result := W5100.StartSPI(0,1,2,3,21)  'Re-Create the W5100 Driver at this Object Level
  if result == 0    'Check for success
    return -3

  pauseMSec(800) 'Wait for new COG to load

  'InitAddresses( _block, _macPTR, _gatewayPTR, _subnetPTR, _ipPTR)
  W5100.InitAddresses(true, @MAC_Address[0], @GatewayIP[0], @SubnetMask[0], @IP[0])

  pauseMSec(100)
'  pst.str(string("launching webserver..."))
'  pst.char($0D)
  lock_id := locknew
  return (webserver_cog := (cognew(webServer(time_ptr, debug_ptr), @Stack) + 1))
  

pub UpdateTemp(Node_Address,temp) 'Updates local cache of node temperatures

   '0.010 V/F * 4 = 0.040 V/F
   '5/1024 = 4.8828125 mV/lsb
   '4.8828125/40 = 0.1220703125 F/lsb
   'T = 0.1220703125 * temperature
'  i := math.ffloat(i)
'  i := math.fmul(f_lsb, i)
   Temperature[Node_Address] := temp
    
PUB Stop
  w5100.stopSPI
  if webserver_cog
    cogstop(webserver_cog~ - 1)
  if lock_id <> -1
    lockret(lock_id)  
  
PUB GetIP(ipPtr)
  bytemove(ipPtr, @IP[0], 4)

PUB GetGatewayIP(gatewayPtr)
  bytemove(gatewayPtr, @GatewayIP[0], 4)
  
PUB GetSubnetMask(subnetPtr)
  bytemove(subnetPtr, @SubnetMask[0], 4)
   
PUB GetDNSServer(dnsPtr)
  bytemove(dnsPtr, @DNS_Server[0], 4)
  
PUB GetDHCPServer(dhcpPtr)
  bytemove(dhcpPtr, @DHCP_Server[0], 4)

PUB GetDomainName
  return @DomainName[0]
PUB GetLeaseTime
  return DHCP_LeaseTime
PUB DHCP_OK
  return DHCP_Success

PUB renewDHCPlease(Time_ptr) | packetSize, serverReply, LoopCnt, socket, skt_status, startcnt, timeoutCnt, currentCnt
  repeat until ina[8] == 1
  repeat until not lockset(lock_id)

'  socket := 0
{  repeat 4 'Scan for first unused/open socket
     w5100.ReadSPI((W5100#_S0_SR + ($0100)*socket),@skt_status,1)

     if (skt_status.byte[0] <> $00) 'Check socket status; if status == 0 then socket is in closed state and is available for use 
       quit
     elseif socket == 3  'If we have looped through all 4 sockets and none are available then exit routine
       return DHCP#SOCKET_FAILED_OPENING

     socket += 1
}
  destIP[0] := 255
  destIP[1] := 255
  destIP[2] := 255
  destIP[3] := 255'}
  W5100.ReadSPI(W5100#_S0_SR, @skt_status,1)
  if skt_status <> W5100#_SOCK_CLOSED
    W5100.SocketClose(0)
    socket := true
  else
    socket := false
          
  W5100.SocketOpen(0, W5100#_UDPPROTO, DHCP#DHCP_CLIENT_PORT, DHCP#DHCP_CLIENT_PORT, @DestIP[0])
  ' check the status of the socket
  w5100.ReadSPI(W5100#_S0_SR ,@skt_status,1)

  if skt_status <> W5100#_SOCK_UDP
     return DHCP#SOCKET_FAILED_OPENING

  bytefill(@Buffer[0],0,Buffer_size)   
  DHCP.FillRenewLeasePacket(@Buffer[0], $4242_4242, @DestIP[0])
  W5100.txUDP(0, @Buffer[0])

  PauseMSec(200)
'  LoopCnt := 0
  ' wait for a DHCP Ack reply, this grants us our claim on the IP
  startCnt := CNT
  timeoutCnt := startCnt + DHCP#DHCP_TIMEOUT
  repeat
    packetSize := W5100.rxUDP(0, @Buffer[0])
    if packetSize > 0
      serverReply := DHCP.ParseDHCPReply($4242_4242, DHCP#DHCP_ACK, @Buffer[0])
      case serverReply
        1: ' we got our packet
          QUIT
        2: ' got NAK from server
          W5100.SocketClose(0)
          SetDefaultConfig
          return DHCP#SERVER_NAK
'        other:
'          return serverReply
'    loopCnt += 1
'    if LoopCnt == 10
'      return DHCP#SERVER_TIMEOUT         
    ' check for timeout, handling wrap case        
    currentCnt := CNT          
    if startCnt < timeoutCnt 
      if (currentCnt => timeoutCnt) OR (currentCnt < startCnt)
        SetDefaultConfig
        return DHCP#SERVER_TIMEOUT
    else
      if (currentCnt => timeoutCnt) AND (currentCnt < startCnt)
        SetDefaultConfig
        return DHCP#SERVER_TIMEOUT
'} 

  
  W5100.SocketClose(0)

  DHCP.getIP(@DestIP[0])
'  repeat LoopCnt from 0 to 3
    if long[@DestIP[0]] <> long[@IP[0]]
      DHCP.getSubnetMask(@SubnetMask[0])
      DHCP.GetGatewayIP(@GatewayIP[0])
      DHCP.GetDNSServer(@DNS_Server[0])
      DHCP.GetDHCPServer(@DHCP_Server[0])
      bytemove(@DomainName[0],DHCP.GetDomainName,strsize(DHCP.GetDomainName))
      bytemove(@IP[0],@DestIP[0],4)
      W5100.SocketClose(0)
      W5100.SocketClose(1)
      W5100.SocketClose(2)
      W5100.SocketClose(3)
'      W5100.ResetHardware(true)
      W5100.InitAddresses(true, @MAC_Address[0], @GatewayIP[0], @SubnetMask[0], @IP[0])
      

  DHCP_LeaseTime := DHCP.GetLeaseTime
  DHCP_LeaseTime += long[Time_Ptr]        

  if socket == true
    OpenSocketAgain
  lockclr(lock_id)
  return DHCP#SUCCESS
  
pri SetDefaultConfig
        GatewayIP[0] := 192
        GatewayIP[1] := 168
        GatewayIP[2] := 0
        GatewayIP[3] := 1
        Subnetmask[0] := $FF
        Subnetmask[1] := $FF
        Subnetmask[2] := $FF
        Subnetmask[3] := 0
        IP[0] := 192
        IP[1] := 168
        IP[2] := 0
        IP[3] := 116
        DHCP_Success := -1
        DHCP_LeaseTime := -1
        
pri obtain_IP(Time_Ptr) | failcount
  failcount := 0
  if DHCP_Success < 0
    return (-1)
  DHCP.sethostname(string("Propeller"))   
  repeat
      PauseMSec(2000)

      result := DHCP.DoDHCP(0, $0102_0304)
    ' at this point if result == SUCCESS, then the W5100 is configured
      if result == DHCP#SUCCESS
        DHCP.getIP(@IP[0])
        DHCP.getSubnetMask(@SubnetMask[0])
        DHCP.GetGatewayIP(@GatewayIP[0])
        DHCP.GetDNSServer(@DNS_Server[0])
        DHCP.GetDHCPServer(@DHCP_Server[0])
        DHCP_LeaseTime := DHCP.GetLeaseTime
        DHCP_LeaseTime += long[Time_Ptr]        
        bytemove(@DomainName[0],DHCP.GetDomainName,strsize(DHCP.GetDomainName))
        return 1
      else
        failcount += 1
        DHCP.stop
        PauseMSec(500)
        DHCP.start(@MAC_Address[0], @Buffer[0])
      if failcount == 10
        dhcp.stop
        return (-1)
pub webServer(time_ptr, Debug_ptr) | temp0, packetSize, startCnt, timeoutCnt, currentCnt, InterruptMask, temp1

  temp0 := 0
  packetSize := 0
  times_served := 0
  
'  pst.str(string("Webserver Starting..."))
'  pst.char($0D)
  
  'Local socket
  localSocket := 80 

  'Destination IP address - can be left zeros, the TCO demo echoes to computer that sent the packet
  destIP[0] := 0
  destIP[1] := 0
  destIP[2] := 0
  destIP[3] := 0

  destSocket := 80

  W5100.SocketOpen(0, W5100#_TCPPROTO, localSocket, destSocket, @destIP[0])

  'Wait a moment for the socket to get established
  PauseMSec(500)

  W5100.readSPI(W5100#_S0_SR, @temp0, 1)

  if temp0 <> W5100#_SOCK_INIT
'      PST.Str(string("Error - invalid status read",pst#NL))
'      pst.dec(temp0)
      return

{  case temp0
    W5100#_SOCK_CLOSED : PST.Str(string("$00 - socket closed", PST#NL, PST#NL))
    W5100#_SOCK_INIT   : PST.Str(string("$13 - socket initalized/opened", PST#NL, PST#NL))
    W5100#_SOCK_LISTEN : PST.Str(string("$14 - socket listening", PST#NL, PST#NL))
    W5100#_SOCK_ESTAB  : PST.Str(string("$17 - socket established", PST#NL, PST#NL))    
    W5100#_SOCK_UDP    : PST.Str(string("$22 - socket UDP open", PST#NL, PST#NL))
}
  W5100.SocketTCPlisten(0)

  'Wait a moment for the socket to listen
  PauseMSec(500)

  W5100.readSPI(W5100#_S0_SR, @temp0, 1)

  if temp0 <> W5100#_SOCK_LISTEN
'      PST.Str(string("Error - invalid status read",pst#NL))
'      pst.dec(temp0)
      return

{  case temp0
    W5100#_SOCK_CLOSED : PST.Str(string("$00 - socket closed", PST#NL, PST#NL))
    W5100#_SOCK_INIT   : PST.Str(string("$13 - socket initalized", PST#NL, PST#NL))
    W5100#_SOCK_LISTEN : PST.Str(string("$14 - socket listening", PST#NL, PST#NL))
    W5100#_SOCK_ESTAB  : PST.Str(string("$17 - socket established", PST#NL, PST#NL))    
    W5100#_SOCK_UDP    : PST.Str(string("$22 - socket UDP open", PST#NL, PST#NL))
}
  'Enable W5100 Interrupts on Socket 0
  dira[8] := w5100#_input
  W5100.ReadSPI(W5100#_IMR, @InterruptMask, 1)'Get current Interrupt mask
  InterruptMask |= 1  'Turn on Socket 0 interrupt
  w5100.WriteSPI(true, W5100#_IMR, @interruptMask, 1) 'Write new Interrupt mask to W5100

  repeat

'    if ! ina[8]

    repeat  'Wait for an interrupt to occur
      long[debug_ptr] := 0
      W5100.ReadSPI(W5100#_S0_SR, @temp1,1)
      if temp1 <> $14
        W5100.socketClose(0)
        OpenSocketAgain
      waitpeq(0, |< 8, 0) 'Conserve power by halting until the INT pin goes low
      repeat until not lockset(lock_id)
      W5100.ReadSPI(W5100#_IR, @InterruptMask, 1) 'Get source of interrupt
      if InterruptMask & $01  'If socket 0 interrupt
        W5100.ReadSPI(W5100#_S0_IR, @InterruptMask, 1) 'Get Socket Interrupt Status
        W5100.WriteSPI(true, W5100#_S0_IR,@InterruptMask, 1) 'Clear Socket Interrupt; This should drive INT pin back High unless other interrupt sources are enabled.
        W5100.ReadSPI(W5100#_S0_SR, @temp1,1) 'Read Socket Status.
        byte[debug_ptr] := temp1.byte[0]
        if temp1 == $1C 'Connection Close Request Received?
             'End the connection
          'W5100.SocketTCPdisconnect(0)

          'PauseMSec(10)

         'Connection terminated
          W5100.SocketClose(0)
        'Once the connection is closed, need to open socket again
          OpenSocketAgain
        elseif temp1 == $17 'We only care about Connection Established Interrupts
          quit  'If connected continue
      else
        InterruptMask := $F0
        W5100.WriteSPI(true, W5100#_IR, @InterruptMask,1)          
      lockclr(lock_id)

    'Testing Socket 0's status register and looking for a client to connect to our server
'    repeat while !W5100.SocketTCPestablished(0)
'      pauseMSec(10)
'    PST.Str(string("connection established..."))
'    temp_wanted := 1
'    repeat until (temp_wanted <> 1)
    
    'Initialize the buffers and bring the data over
    bytefill(@Buffer, 0, BUFFER_SIZE)    
'    W5100.rxTCP(0, @Buffer)

    startCnt := CNT    
    timeoutCnt := startCnt + HTTP_TIMEOUT
    repeat
      pkt := packetSize := W5100.rxTCP(0, @Buffer)
      currentCnt := CNT          
      if startCnt < timeoutCnt 
        if (currentCnt => timeoutCnt) OR (currentCnt < startCnt)
          quit
      else
        if (currentCnt => timeoutCnt) AND (currentCnt < startCnt)
          quit
        
    while packetSize == 0

    if packetSize > 0   
      ParseRequest
      'Send the web page - hardcoded here
      'status lin
      if RequestType == HTTP_REQUEST_GET
        servePage(Time_ptr,temp1)
       
    PauseMSec(5)

   'End the connection
    W5100.SocketTCPdisconnect(0)

    PauseMSec(10)

   'Connection terminated
    W5100.SocketClose(0)
   'Once the connection is closed, need to open socket again
    OpenSocketAgain
    lockclr(lock_id)
PRI StringSend(_socket, _dataPtr)
'***************************************

  W5100.txTCP(0, _dataPtr, strsize(_dataPtr))

  return 'end of StringSend

'***************************************
PRI OpenSocketAgain
'***************************************
  pauseMSec(100)
  W5100.SocketOpen(0, W5100#_TCPPROTO, localSocket, destSocket, @destIP[0])
  pauseMSec(100)
  W5100.SocketTCPlisten(0)
  pauseMSec(100)  

  return 'end of OpenSocketAgain

PRI ParseRequest | index, char, inPlace, URI, requestEnd, decodeNum, token  ' !!! NOTICE - THIS MODIFIES data[] !!!

  RequestType := HTTP_REQUEST_HEAD

  requestEnd := @Buffer[0] + strsize(@Buffer[0])

  ' parse the request type
  'token := STR.tokenizeCharacters(@Buffer[0])
  token := STR.trimCharacters(@Buffer[0])
  'STR.CharactersToUpperCase(token)
  if STR.startsWithCharacters(token, string("GET / HTTP/1.1"))
     RequestType := HTTP_REQUEST_GET
     
'  elseif STR.startsWithCharacters(token, string("HEAD"))
'    RequestType := HTTP_REQUEST_HEAD
'  elseif STR.startsWithCharacters(token, string("POST"))
'    RequestType := HTTP_REQUEST_POST

pri ServePage(time_ptr,debug_Val) | temp1, temp2
        times_served += 1
        temp1 := ConvertToTemp(1)
        temp2 := ConvertToTemp(2)
        StringSend(0, string("HTTP/1.1 200 OK"))
        StringSend(0, string(NL, LF))
       
        'optional header
        StringSend(0, string("Server: Parallax Propeller + WIZ811MJ Web Server/demo 1"))
        StringSend(0, string(" Connection: close"))
        StringSend(0, string(" Content-Type: text/html"))
        StringSend(0, string(NL, LF))
       
      'blank line
        StringSend(0, string(NL, LF))
       
      'File
        StringSend(0, string("<HTML>", NL))
        StringSend(0, string("<HEAD>", NL))
        StringSend(0, string("<TITLE>"))
        StringSend(0, string("Shawn Loves Kelly"))
        StringSend(0, string("</TITLE>", NL))
        StringSend(0, string("</HEAD>", NL))
        StringSend(0, string("<BODY>", NL))
        StringSend(0, string("<H1>"))
        StringSend(0, string("Helloooooo Kelly!!"))
        StringSend(0, string("</H1>", NL))
        StringSend(0, string("<HR>", NL))
        StringSend(0, string("<P>"))  
        StringSend(0, string("This page is brought to you by the Parallax Propeller (an 8 core microcontroller) + WizNet W5100 TCP/IP Stack"))
        StringSend(0, string("</P>", NL))
        StringSend(0, string("<P>"))  
        StringSend(0, string("Debug Value: "))
        StringSend(0, STR.NumbertoDecimal(Debug_Val,4))
        StringSend(0, string("</P>", NL))
        StringSend(0, string("<P>", NL))
        StringSend(0, string("Server Up-Time (seconds): "))
        StringSend(0, STR.NumbertoDecimal(long[Time_ptr],10))
        StringSend(0, string(NL, "</P>", NL, "<P>"))
        StringSend(0, string("Master Bedroom Temperature is: "))
        StringSend(0, STR.NumberToDecimal(Temp1,3))
        StringSend(0, string("</P>",NL,"<P>"))
        StringSend(0, string("Living Room Temperature is: "))
        StringSend(0, STR.NumberToDecimal(Temp2,3))
        StringSend(0, string(NL,"</P>",NL,"<P>"))
        StringSend(0, string("Site Counter: "))
        StringSend(0, STR.NumberToDecimal(times_served,5))
        StringSend(0, string(NL,"</P>",NL,"<P> Your HTTP Request Packet looks like: ","<BR>"))
        StringSend(0, @Buffer[0])
        StringSend(0, string("</P>"))
        StringSend(0, string("</BODY>", NL))
        StringSend(0, string("</HTML>", NL))
        StringSend(0, string(NL, LF))
'}
pri ConvertToTemp(address) | i

 '0.010 V/F * 4 = 0.040 V/F
 '5/1024 = 4.8828125 mV/lsb
 '4.8828125/40 = 0.1220703125 F/lsb
 'T = 0.1220703125 * temperature
   i := Temperature[address]
   i := math.ffloat(i)
   i := math.fmul(f_lsb, i)
   i := math.fround(i)
 return i
  
pub GetTemp(param)
  if param == 0
    return temp_wanted
  elseif param == -1
    temp_wanted := 0
PUB GetPageCount
  return times_served    
pri PauseMSec(Duration)
''  Pause execution for specified milliseconds.
''  This routine is based on the set clock frequency.
''  
''  params:  Duration = number of milliseconds to delay                                                                                               
''  return:  none
  waitcnt(((clkfreq / 1_000 * Duration - 3932) #> 381) + cnt)

PUB getHTTP
  return @buffer[0]
PUB getPacketSize
  return pkt
  
var

  long pkt    